# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1 import (
    Subscription,
    SubscriptionListResponse,
    SubscriptionImportResponse,
    SubscriptionPreviewResponse,
    SubscriptionProvisionResponse,
)
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSubscriptions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.retrieve(
            "x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.update(
            id="x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.update(
            id="x",
            addons=[
                {
                    "id": "id",
                    "quantity": 0,
                }
            ],
            applied_coupon={
                "billing_coupon_id": "billingCouponId",
                "configuration": {"start_date": parse_datetime("2019-12-27T18:11:19.117Z")},
                "coupon_id": "couponId",
                "discount": {
                    "amounts_off": [
                        {
                            "amount": 0,
                            "currency": "usd",
                        }
                    ],
                    "description": "description",
                    "duration_in_months": 1,
                    "name": "name",
                    "percent_off": 1,
                },
                "promotion_code": "promotionCode",
            },
            await_payment_confirmation=True,
            billing_cycle_anchor="UNCHANGED",
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "charge_on_behalf_of_account": "chargeOnBehalfOfAccount",
                "coupon_id": "couponId",
                "integration_id": "integrationId",
                "invoice_days_until_due": 0,
                "is_backdated": True,
                "is_invoice_paid": True,
                "metadata": {"foo": "bar"},
                "proration_behavior": "INVOICE_IMMEDIATELY",
                "tax_ids": [
                    {
                        "type": "type",
                        "value": "value",
                    }
                ],
                "tax_percentage": 0,
                "tax_rate_ids": ["string"],
            },
            billing_period="MONTHLY",
            budget={
                "has_soft_limit": True,
                "limit": 0,
            },
            charges=[
                {
                    "id": "id",
                    "quantity": 1,
                    "type": "FEATURE",
                }
            ],
            metadata={"foo": "string"},
            minimum_spend={
                "amount": 0,
                "currency": "usd",
            },
            price_overrides=[
                {
                    "addon_id": "addonId",
                    "amount": 0,
                    "base_charge": True,
                    "currency": "usd",
                    "currency_id": "currencyId",
                    "feature_id": "featureId",
                }
            ],
            promotion_code="promotionCode",
            schedule_strategy="END_OF_BILLING_PERIOD",
            subscription_entitlements=[
                {
                    "id": "id",
                    "feature_id": "featureId",
                    "has_soft_limit": True,
                    "has_unlimited_usage": True,
                    "monthly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "reset_period": "YEAR",
                    "usage_limit": 0,
                    "weekly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "yearly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                }
            ],
            trial_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.update(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.update(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.list()
        assert_matches_type(SyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.list(
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            customer_id="customerId",
            limit=1,
            plan_id="planId",
            pricing_type="pricingType",
            resource_id="resourceId",
            status="status",
        )
        assert_matches_type(SyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_cancel(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.cancel(
            id="x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_cancel_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.cancel(
            id="x",
            cancellation_action="DEFAULT",
            cancellation_time="END_OF_BILLING_PERIOD",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            prorate=True,
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_cancel(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.cancel(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_cancel(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.cancel(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_cancel(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.with_raw_response.cancel(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delegate(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.delegate(
            id="x",
            target_customer_id="targetCustomerId",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delegate(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.delegate(
            id="x",
            target_customer_id="targetCustomerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delegate(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.delegate(
            id="x",
            target_customer_id="targetCustomerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delegate(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.with_raw_response.delegate(
                id="",
                target_customer_id="targetCustomerId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_import(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                }
            ],
        )
        assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_import_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                    "addons": [
                        {
                            "id": "id",
                            "quantity": 0,
                        }
                    ],
                    "billing_id": "billingId",
                    "billing_period": "MONTHLY",
                    "charges": [
                        {
                            "id": "id",
                            "quantity": 1,
                            "type": "FEATURE",
                        }
                    ],
                    "end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "metadata": {"foo": "string"},
                    "resource_id": "resourceId",
                    "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            integration_id="integrationId",
        )
        assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_import(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_import(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_migrate(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.migrate(
            id="x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_migrate_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.migrate(
            id="x",
            subscription_migration_time="END_OF_BILLING_PERIOD",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_migrate(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.migrate(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_migrate(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.migrate(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_migrate(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.with_raw_response.migrate(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_preview(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.preview(
            customer_id="customerId",
            plan_id="planId",
        )
        assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_preview_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.preview(
            customer_id="customerId",
            plan_id="planId",
            addons=[
                {
                    "id": "id",
                    "quantity": 0,
                }
            ],
            applied_coupon={
                "billing_coupon_id": "billingCouponId",
                "configuration": {"start_date": parse_datetime("2019-12-27T18:11:19.117Z")},
                "coupon_id": "couponId",
                "discount": {
                    "amounts_off": [
                        {
                            "amount": 0,
                            "currency": "usd",
                        }
                    ],
                    "description": "description",
                    "duration_in_months": 1,
                    "name": "name",
                    "percent_off": 1,
                },
                "promotion_code": "promotionCode",
            },
            billable_features=[
                {
                    "feature_id": "featureId",
                    "quantity": 1,
                }
            ],
            billing_country_code="billingCountryCode",
            billing_cycle_anchor="UNCHANGED",
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "charge_on_behalf_of_account": "chargeOnBehalfOfAccount",
                "integration_id": "integrationId",
                "invoice_days_until_due": 0,
                "is_backdated": True,
                "is_invoice_paid": True,
                "metadata": {},
                "proration_behavior": "INVOICE_IMMEDIATELY",
                "tax_ids": [
                    {
                        "type": "type",
                        "value": "value",
                    }
                ],
                "tax_percentage": 0,
                "tax_rate_ids": ["string"],
            },
            billing_period="MONTHLY",
            charges=[
                {
                    "id": "id",
                    "quantity": 1,
                    "type": "FEATURE",
                }
            ],
            paying_customer_id="payingCustomerId",
            resource_id="resourceId",
            schedule_strategy="END_OF_BILLING_PERIOD",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            trial_override_configuration={
                "is_trial": True,
                "trial_end_behavior": "CONVERT_TO_PAID",
                "trial_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            unit_quantity=1,
        )
        assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_preview(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.preview(
            customer_id="customerId",
            plan_id="planId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_preview(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.preview(
            customer_id="customerId",
            plan_id="planId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_provision(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.provision(
            customer_id="customerId",
            plan_id="planId",
        )
        assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_provision_with_all_params(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.provision(
            customer_id="customerId",
            plan_id="planId",
            id="id",
            addons=[
                {
                    "id": "id",
                    "quantity": 0,
                }
            ],
            applied_coupon={
                "billing_coupon_id": "billingCouponId",
                "configuration": {"start_date": parse_datetime("2019-12-27T18:11:19.117Z")},
                "coupon_id": "couponId",
                "discount": {
                    "amounts_off": [
                        {
                            "amount": 0,
                            "currency": "usd",
                        }
                    ],
                    "description": "description",
                    "duration_in_months": 1,
                    "name": "name",
                    "percent_off": 1,
                },
                "promotion_code": "promotionCode",
            },
            await_payment_confirmation=True,
            billing_country_code="billingCountryCode",
            billing_cycle_anchor="UNCHANGED",
            billing_id="billingId",
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "charge_on_behalf_of_account": "chargeOnBehalfOfAccount",
                "integration_id": "integrationId",
                "invoice_days_until_due": 0,
                "is_backdated": True,
                "is_invoice_paid": True,
                "metadata": {"foo": "string"},
                "proration_behavior": "INVOICE_IMMEDIATELY",
                "tax_ids": [
                    {
                        "type": "type",
                        "value": "value",
                    }
                ],
                "tax_percentage": 0,
                "tax_rate_ids": ["string"],
            },
            billing_period="MONTHLY",
            budget={
                "has_soft_limit": True,
                "limit": 0,
            },
            charges=[
                {
                    "id": "id",
                    "quantity": 1,
                    "type": "FEATURE",
                }
            ],
            checkout_options={
                "cancel_url": "https://example.com",
                "success_url": "https://example.com",
                "allow_promo_codes": True,
                "allow_tax_id_collection": True,
                "collect_billing_address": True,
                "collect_phone_number": True,
                "reference_id": "referenceId",
            },
            metadata={"foo": "string"},
            minimum_spend={
                "amount": 0,
                "currency": "usd",
            },
            paying_customer_id="payingCustomerId",
            payment_collection_method="CHARGE",
            price_overrides=[
                {
                    "addon_id": "addonId",
                    "amount": 0,
                    "base_charge": True,
                    "billing_country_code": "billingCountryCode",
                    "block_size": 0,
                    "credit_grant_cadence": "BEGINNING_OF_BILLING_PERIOD",
                    "credit_rate": {
                        "amount": 1,
                        "currency_id": "currencyId",
                        "cost_formula": "costFormula",
                    },
                    "currency": "usd",
                    "feature_id": "featureId",
                    "tiers": [
                        {
                            "flat_price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "unit_price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "up_to": 0,
                        }
                    ],
                }
            ],
            resource_id="resourceId",
            salesforce_id="salesforceId",
            schedule_strategy="END_OF_BILLING_PERIOD",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            subscription_entitlements=[
                {
                    "feature_id": "featureId",
                    "usage_limit": 0,
                    "is_granted": True,
                }
            ],
            trial_override_configuration={
                "is_trial": True,
                "trial_end_behavior": "CONVERT_TO_PAID",
                "trial_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            unit_quantity=1,
        )
        assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_provision(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.provision(
            customer_id="customerId",
            plan_id="planId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_provision(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.provision(
            customer_id="customerId",
            plan_id="planId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_transfer(self, client: Stigg) -> None:
        subscription = client.v1.subscriptions.transfer(
            id="x",
            destination_resource_id="destinationResourceId",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_transfer(self, client: Stigg) -> None:
        response = client.v1.subscriptions.with_raw_response.transfer(
            id="x",
            destination_resource_id="destinationResourceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_transfer(self, client: Stigg) -> None:
        with client.v1.subscriptions.with_streaming_response.transfer(
            id="x",
            destination_resource_id="destinationResourceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_transfer(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.subscriptions.with_raw_response.transfer(
                id="",
                destination_resource_id="destinationResourceId",
            )


class TestAsyncSubscriptions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.retrieve(
            "x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.update(
            id="x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.update(
            id="x",
            addons=[
                {
                    "id": "id",
                    "quantity": 0,
                }
            ],
            applied_coupon={
                "billing_coupon_id": "billingCouponId",
                "configuration": {"start_date": parse_datetime("2019-12-27T18:11:19.117Z")},
                "coupon_id": "couponId",
                "discount": {
                    "amounts_off": [
                        {
                            "amount": 0,
                            "currency": "usd",
                        }
                    ],
                    "description": "description",
                    "duration_in_months": 1,
                    "name": "name",
                    "percent_off": 1,
                },
                "promotion_code": "promotionCode",
            },
            await_payment_confirmation=True,
            billing_cycle_anchor="UNCHANGED",
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "charge_on_behalf_of_account": "chargeOnBehalfOfAccount",
                "coupon_id": "couponId",
                "integration_id": "integrationId",
                "invoice_days_until_due": 0,
                "is_backdated": True,
                "is_invoice_paid": True,
                "metadata": {"foo": "bar"},
                "proration_behavior": "INVOICE_IMMEDIATELY",
                "tax_ids": [
                    {
                        "type": "type",
                        "value": "value",
                    }
                ],
                "tax_percentage": 0,
                "tax_rate_ids": ["string"],
            },
            billing_period="MONTHLY",
            budget={
                "has_soft_limit": True,
                "limit": 0,
            },
            charges=[
                {
                    "id": "id",
                    "quantity": 1,
                    "type": "FEATURE",
                }
            ],
            metadata={"foo": "string"},
            minimum_spend={
                "amount": 0,
                "currency": "usd",
            },
            price_overrides=[
                {
                    "addon_id": "addonId",
                    "amount": 0,
                    "base_charge": True,
                    "currency": "usd",
                    "currency_id": "currencyId",
                    "feature_id": "featureId",
                }
            ],
            promotion_code="promotionCode",
            schedule_strategy="END_OF_BILLING_PERIOD",
            subscription_entitlements=[
                {
                    "id": "id",
                    "feature_id": "featureId",
                    "has_soft_limit": True,
                    "has_unlimited_usage": True,
                    "monthly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "reset_period": "YEAR",
                    "usage_limit": 0,
                    "weekly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "yearly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                }
            ],
            trial_end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.update(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.update(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.list()
        assert_matches_type(AsyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.list(
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            customer_id="customerId",
            limit=1,
            plan_id="planId",
            pricing_type="pricingType",
            resource_id="resourceId",
            status="status",
        )
        assert_matches_type(AsyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[SubscriptionListResponse], subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_cancel(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.cancel(
            id="x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_cancel_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.cancel(
            id="x",
            cancellation_action="DEFAULT",
            cancellation_time="END_OF_BILLING_PERIOD",
            end_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            prorate=True,
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.cancel(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.cancel(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.with_raw_response.cancel(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delegate(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.delegate(
            id="x",
            target_customer_id="targetCustomerId",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delegate(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.delegate(
            id="x",
            target_customer_id="targetCustomerId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delegate(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.delegate(
            id="x",
            target_customer_id="targetCustomerId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delegate(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.with_raw_response.delegate(
                id="",
                target_customer_id="targetCustomerId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_import(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                }
            ],
        )
        assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_import_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                    "addons": [
                        {
                            "id": "id",
                            "quantity": 0,
                        }
                    ],
                    "billing_id": "billingId",
                    "billing_period": "MONTHLY",
                    "charges": [
                        {
                            "id": "id",
                            "quantity": 1,
                            "type": "FEATURE",
                        }
                    ],
                    "end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "metadata": {"foo": "string"},
                    "resource_id": "resourceId",
                    "start_date": parse_datetime("2019-12-27T18:11:19.117Z"),
                }
            ],
            integration_id="integrationId",
        )
        assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_import(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_import(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.import_(
            subscriptions=[
                {
                    "id": "id",
                    "customer_id": "customerId",
                    "plan_id": "planId",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionImportResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_migrate(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.migrate(
            id="x",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_migrate_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.migrate(
            id="x",
            subscription_migration_time="END_OF_BILLING_PERIOD",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_migrate(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.migrate(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_migrate(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.migrate(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_migrate(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.with_raw_response.migrate(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_preview(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.preview(
            customer_id="customerId",
            plan_id="planId",
        )
        assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_preview_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.preview(
            customer_id="customerId",
            plan_id="planId",
            addons=[
                {
                    "id": "id",
                    "quantity": 0,
                }
            ],
            applied_coupon={
                "billing_coupon_id": "billingCouponId",
                "configuration": {"start_date": parse_datetime("2019-12-27T18:11:19.117Z")},
                "coupon_id": "couponId",
                "discount": {
                    "amounts_off": [
                        {
                            "amount": 0,
                            "currency": "usd",
                        }
                    ],
                    "description": "description",
                    "duration_in_months": 1,
                    "name": "name",
                    "percent_off": 1,
                },
                "promotion_code": "promotionCode",
            },
            billable_features=[
                {
                    "feature_id": "featureId",
                    "quantity": 1,
                }
            ],
            billing_country_code="billingCountryCode",
            billing_cycle_anchor="UNCHANGED",
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "charge_on_behalf_of_account": "chargeOnBehalfOfAccount",
                "integration_id": "integrationId",
                "invoice_days_until_due": 0,
                "is_backdated": True,
                "is_invoice_paid": True,
                "metadata": {},
                "proration_behavior": "INVOICE_IMMEDIATELY",
                "tax_ids": [
                    {
                        "type": "type",
                        "value": "value",
                    }
                ],
                "tax_percentage": 0,
                "tax_rate_ids": ["string"],
            },
            billing_period="MONTHLY",
            charges=[
                {
                    "id": "id",
                    "quantity": 1,
                    "type": "FEATURE",
                }
            ],
            paying_customer_id="payingCustomerId",
            resource_id="resourceId",
            schedule_strategy="END_OF_BILLING_PERIOD",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            trial_override_configuration={
                "is_trial": True,
                "trial_end_behavior": "CONVERT_TO_PAID",
                "trial_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            unit_quantity=1,
        )
        assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_preview(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.preview(
            customer_id="customerId",
            plan_id="planId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_preview(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.preview(
            customer_id="customerId",
            plan_id="planId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionPreviewResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_provision(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.provision(
            customer_id="customerId",
            plan_id="planId",
        )
        assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_provision_with_all_params(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.provision(
            customer_id="customerId",
            plan_id="planId",
            id="id",
            addons=[
                {
                    "id": "id",
                    "quantity": 0,
                }
            ],
            applied_coupon={
                "billing_coupon_id": "billingCouponId",
                "configuration": {"start_date": parse_datetime("2019-12-27T18:11:19.117Z")},
                "coupon_id": "couponId",
                "discount": {
                    "amounts_off": [
                        {
                            "amount": 0,
                            "currency": "usd",
                        }
                    ],
                    "description": "description",
                    "duration_in_months": 1,
                    "name": "name",
                    "percent_off": 1,
                },
                "promotion_code": "promotionCode",
            },
            await_payment_confirmation=True,
            billing_country_code="billingCountryCode",
            billing_cycle_anchor="UNCHANGED",
            billing_id="billingId",
            billing_information={
                "billing_address": {
                    "city": "city",
                    "country": "country",
                    "line1": "line1",
                    "line2": "line2",
                    "postal_code": "postalCode",
                    "state": "state",
                },
                "charge_on_behalf_of_account": "chargeOnBehalfOfAccount",
                "integration_id": "integrationId",
                "invoice_days_until_due": 0,
                "is_backdated": True,
                "is_invoice_paid": True,
                "metadata": {"foo": "string"},
                "proration_behavior": "INVOICE_IMMEDIATELY",
                "tax_ids": [
                    {
                        "type": "type",
                        "value": "value",
                    }
                ],
                "tax_percentage": 0,
                "tax_rate_ids": ["string"],
            },
            billing_period="MONTHLY",
            budget={
                "has_soft_limit": True,
                "limit": 0,
            },
            charges=[
                {
                    "id": "id",
                    "quantity": 1,
                    "type": "FEATURE",
                }
            ],
            checkout_options={
                "cancel_url": "https://example.com",
                "success_url": "https://example.com",
                "allow_promo_codes": True,
                "allow_tax_id_collection": True,
                "collect_billing_address": True,
                "collect_phone_number": True,
                "reference_id": "referenceId",
            },
            metadata={"foo": "string"},
            minimum_spend={
                "amount": 0,
                "currency": "usd",
            },
            paying_customer_id="payingCustomerId",
            payment_collection_method="CHARGE",
            price_overrides=[
                {
                    "addon_id": "addonId",
                    "amount": 0,
                    "base_charge": True,
                    "billing_country_code": "billingCountryCode",
                    "block_size": 0,
                    "credit_grant_cadence": "BEGINNING_OF_BILLING_PERIOD",
                    "credit_rate": {
                        "amount": 1,
                        "currency_id": "currencyId",
                        "cost_formula": "costFormula",
                    },
                    "currency": "usd",
                    "feature_id": "featureId",
                    "tiers": [
                        {
                            "flat_price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "unit_price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "up_to": 0,
                        }
                    ],
                }
            ],
            resource_id="resourceId",
            salesforce_id="salesforceId",
            schedule_strategy="END_OF_BILLING_PERIOD",
            start_date=parse_datetime("2019-12-27T18:11:19.117Z"),
            subscription_entitlements=[
                {
                    "feature_id": "featureId",
                    "usage_limit": 0,
                    "is_granted": True,
                }
            ],
            trial_override_configuration={
                "is_trial": True,
                "trial_end_behavior": "CONVERT_TO_PAID",
                "trial_end_date": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            unit_quantity=1,
        )
        assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_provision(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.provision(
            customer_id="customerId",
            plan_id="planId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_provision(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.provision(
            customer_id="customerId",
            plan_id="planId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(SubscriptionProvisionResponse, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_transfer(self, async_client: AsyncStigg) -> None:
        subscription = await async_client.v1.subscriptions.transfer(
            id="x",
            destination_resource_id="destinationResourceId",
        )
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_transfer(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.subscriptions.with_raw_response.transfer(
            id="x",
            destination_resource_id="destinationResourceId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscription = await response.parse()
        assert_matches_type(Subscription, subscription, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_transfer(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.subscriptions.with_streaming_response.transfer(
            id="x",
            destination_resource_id="destinationResourceId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscription = await response.parse()
            assert_matches_type(Subscription, subscription, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_transfer(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.subscriptions.with_raw_response.transfer(
                id="",
                destination_resource_id="destinationResourceId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = [
    "SubscriptionProvisionParams",
    "Addon",
    "AppliedCoupon",
    "AppliedCouponConfiguration",
    "AppliedCouponDiscount",
    "AppliedCouponDiscountAmountsOff",
    "BillingInformation",
    "BillingInformationBillingAddress",
    "BillingInformationTaxID",
    "Budget",
    "Charge",
    "CheckoutOptions",
    "MinimumSpend",
    "PriceOverride",
    "PriceOverrideCreditRate",
    "PriceOverrideTier",
    "PriceOverrideTierFlatPrice",
    "PriceOverrideTierUnitPrice",
    "SubscriptionEntitlement",
    "TrialOverrideConfiguration",
]


class SubscriptionProvisionParams(TypedDict, total=False):
    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Customer ID to provision the subscription for"""

    plan_id: Required[Annotated[str, PropertyInfo(alias="planId")]]
    """Plan ID to provision"""

    id: str
    """Unique identifier for the subscription"""

    addons: Iterable[Addon]

    applied_coupon: Annotated[AppliedCoupon, PropertyInfo(alias="appliedCoupon")]
    """Coupon configuration"""

    await_payment_confirmation: Annotated[bool, PropertyInfo(alias="awaitPaymentConfirmation")]
    """Whether to wait for payment confirmation before returning the subscription"""

    billing_country_code: Annotated[Optional[str], PropertyInfo(alias="billingCountryCode")]
    """The ISO 3166-1 alpha-2 country code for billing"""

    billing_cycle_anchor: Annotated[Literal["UNCHANGED", "NOW"], PropertyInfo(alias="billingCycleAnchor")]
    """Billing cycle anchor behavior for the subscription"""

    billing_id: Annotated[Optional[str], PropertyInfo(alias="billingId")]
    """External billing system identifier"""

    billing_information: Annotated[BillingInformation, PropertyInfo(alias="billingInformation")]

    billing_period: Annotated[Literal["MONTHLY", "ANNUALLY"], PropertyInfo(alias="billingPeriod")]
    """Billing period (MONTHLY or ANNUALLY)"""

    budget: Optional[Budget]

    charges: Iterable[Charge]

    checkout_options: Annotated[CheckoutOptions, PropertyInfo(alias="checkoutOptions")]
    """Checkout page configuration for payment collection"""

    metadata: Dict[str, str]
    """Additional metadata for the subscription"""

    minimum_spend: Annotated[Optional[MinimumSpend], PropertyInfo(alias="minimumSpend")]
    """Minimum spend amount"""

    paying_customer_id: Annotated[Optional[str], PropertyInfo(alias="payingCustomerId")]
    """Optional paying customer ID for split billing scenarios"""

    payment_collection_method: Annotated[
        Literal["CHARGE", "INVOICE", "NONE"], PropertyInfo(alias="paymentCollectionMethod")
    ]
    """How payments should be collected for this subscription"""

    price_overrides: Annotated[Iterable[PriceOverride], PropertyInfo(alias="priceOverrides")]

    resource_id: Annotated[Optional[str], PropertyInfo(alias="resourceId")]
    """Optional resource ID for multi-instance subscriptions"""

    salesforce_id: Annotated[Optional[str], PropertyInfo(alias="salesforceId")]
    """Salesforce ID"""

    schedule_strategy: Annotated[
        Literal["END_OF_BILLING_PERIOD", "END_OF_BILLING_MONTH", "IMMEDIATE"], PropertyInfo(alias="scheduleStrategy")
    ]
    """Strategy for scheduling subscription changes"""

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """Subscription start date"""

    subscription_entitlements: Annotated[
        Iterable[SubscriptionEntitlement], PropertyInfo(alias="subscriptionEntitlements")
    ]

    trial_override_configuration: Annotated[
        TrialOverrideConfiguration, PropertyInfo(alias="trialOverrideConfiguration")
    ]
    """Trial period override settings"""

    unit_quantity: Annotated[float, PropertyInfo(alias="unitQuantity")]


class Addon(TypedDict, total=False):
    """Addon configuration"""

    id: Required[str]
    """Addon ID"""

    quantity: Required[int]
    """Number of addon instances"""


class AppliedCouponConfiguration(TypedDict, total=False):
    """Coupon timing configuration"""

    start_date: Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]
    """Coupon start date"""


class AppliedCouponDiscountAmountsOff(TypedDict, total=False):
    amount: Required[float]
    """The price amount"""

    currency: Required[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ]
    """The price currency"""


class AppliedCouponDiscount(TypedDict, total=False):
    """Ad-hoc discount configuration"""

    amounts_off: Annotated[Optional[Iterable[AppliedCouponDiscountAmountsOff]], PropertyInfo(alias="amountsOff")]
    """Fixed amounts off by currency"""

    description: str
    """Ad-hoc discount"""

    duration_in_months: Annotated[float, PropertyInfo(alias="durationInMonths")]
    """Duration in months"""

    name: str
    """Discount name"""

    percent_off: Annotated[float, PropertyInfo(alias="percentOff")]
    """Percentage discount"""


class AppliedCoupon(TypedDict, total=False):
    """Coupon configuration"""

    billing_coupon_id: Annotated[str, PropertyInfo(alias="billingCouponId")]
    """Billing provider coupon ID"""

    configuration: AppliedCouponConfiguration
    """Coupon timing configuration"""

    coupon_id: Annotated[str, PropertyInfo(alias="couponId")]
    """Stigg coupon ID"""

    discount: AppliedCouponDiscount
    """Ad-hoc discount configuration"""

    promotion_code: Annotated[str, PropertyInfo(alias="promotionCode")]
    """Promotion code to apply"""


class BillingInformationBillingAddress(TypedDict, total=False):
    """Billing address for the subscription"""

    city: str

    country: str

    line1: str

    line2: str

    postal_code: Annotated[str, PropertyInfo(alias="postalCode")]

    state: str


class BillingInformationTaxID(TypedDict, total=False):
    type: Required[str]
    """The type of tax exemption identifier, such as VAT."""

    value: Required[str]
    """The actual tax identifier value"""


class BillingInformation(TypedDict, total=False):
    billing_address: Annotated[BillingInformationBillingAddress, PropertyInfo(alias="billingAddress")]
    """Billing address for the subscription"""

    charge_on_behalf_of_account: Annotated[Optional[str], PropertyInfo(alias="chargeOnBehalfOfAccount")]
    """Stripe Connect account to charge on behalf of"""

    integration_id: Annotated[Optional[str], PropertyInfo(alias="integrationId")]
    """Billing integration identifier"""

    invoice_days_until_due: Annotated[float, PropertyInfo(alias="invoiceDaysUntilDue")]
    """Number of days until invoice is due"""

    is_backdated: Annotated[bool, PropertyInfo(alias="isBackdated")]
    """Whether the subscription is backdated"""

    is_invoice_paid: Annotated[bool, PropertyInfo(alias="isInvoicePaid")]
    """Whether the invoice is marked as paid"""

    metadata: Dict[str, str]
    """Additional metadata for the subscription"""

    proration_behavior: Annotated[
        Literal["INVOICE_IMMEDIATELY", "CREATE_PRORATIONS", "NONE"], PropertyInfo(alias="prorationBehavior")
    ]
    """How to handle proration for billing changes"""

    tax_ids: Annotated[Iterable[BillingInformationTaxID], PropertyInfo(alias="taxIds")]
    """Customer tax identification numbers"""

    tax_percentage: Annotated[float, PropertyInfo(alias="taxPercentage")]
    """Tax percentage (0-100)"""

    tax_rate_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="taxRateIds")]
    """Tax rate identifiers to apply"""


class Budget(TypedDict, total=False):
    has_soft_limit: Required[Annotated[bool, PropertyInfo(alias="hasSoftLimit")]]
    """Whether the budget is a soft limit"""

    limit: Required[float]
    """Maximum spending limit"""


class Charge(TypedDict, total=False):
    """Charge item"""

    id: Required[str]
    """Charge ID"""

    quantity: Required[float]
    """Charge quantity"""

    type: Required[Literal["FEATURE", "CREDIT"]]
    """Charge type"""


class CheckoutOptions(TypedDict, total=False):
    """Checkout page configuration for payment collection"""

    cancel_url: Required[Annotated[str, PropertyInfo(alias="cancelUrl")]]
    """URL to redirect to if checkout is canceled"""

    success_url: Required[Annotated[str, PropertyInfo(alias="successUrl")]]
    """URL to redirect to after successful checkout"""

    allow_promo_codes: Annotated[bool, PropertyInfo(alias="allowPromoCodes")]
    """Allow promotional codes during checkout"""

    allow_tax_id_collection: Annotated[bool, PropertyInfo(alias="allowTaxIdCollection")]
    """Allow tax ID collection during checkout"""

    collect_billing_address: Annotated[bool, PropertyInfo(alias="collectBillingAddress")]
    """Collect billing address during checkout"""

    collect_phone_number: Annotated[bool, PropertyInfo(alias="collectPhoneNumber")]
    """Collect phone number during checkout"""

    reference_id: Annotated[Optional[str], PropertyInfo(alias="referenceId")]
    """Optional reference ID for the checkout session"""


class MinimumSpend(TypedDict, total=False):
    """Minimum spend amount"""

    amount: float
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class PriceOverrideCreditRate(TypedDict, total=False):
    amount: Required[float]
    """The credit rate amount"""

    currency_id: Required[Annotated[str, PropertyInfo(alias="currencyId")]]
    """The custom currency refId for the credit rate"""

    cost_formula: Annotated[Optional[str], PropertyInfo(alias="costFormula")]
    """A custom formula for calculating cost based on single event dimensions"""


class PriceOverrideTierFlatPrice(TypedDict, total=False):
    """The flat fee price of the price tier"""

    amount: Required[float]
    """The price amount"""

    currency: Required[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ]
    """The price currency"""


class PriceOverrideTierUnitPrice(TypedDict, total=False):
    """The unit price of the price tier"""

    amount: Required[float]
    """The price amount"""

    currency: Required[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ]
    """The price currency"""


class PriceOverrideTier(TypedDict, total=False):
    flat_price: Annotated[PriceOverrideTierFlatPrice, PropertyInfo(alias="flatPrice")]
    """The flat fee price of the price tier"""

    unit_price: Annotated[PriceOverrideTierUnitPrice, PropertyInfo(alias="unitPrice")]
    """The unit price of the price tier"""

    up_to: Annotated[float, PropertyInfo(alias="upTo")]
    """The up to quantity of the price tier"""


class PriceOverride(TypedDict, total=False):
    addon_id: Annotated[Optional[str], PropertyInfo(alias="addonId")]
    """Addon identifier for the price override"""

    amount: float
    """The price amount"""

    base_charge: Annotated[bool, PropertyInfo(alias="baseCharge")]
    """Whether this is a base charge override"""

    billing_country_code: Annotated[str, PropertyInfo(alias="billingCountryCode")]
    """The billing country code of the price"""

    block_size: Annotated[float, PropertyInfo(alias="blockSize")]
    """Block size for pricing"""

    credit_grant_cadence: Annotated[
        Literal["BEGINNING_OF_BILLING_PERIOD", "MONTHLY"], PropertyInfo(alias="creditGrantCadence")
    ]

    credit_rate: Annotated[PriceOverrideCreditRate, PropertyInfo(alias="creditRate")]

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""

    feature_id: Annotated[Optional[str], PropertyInfo(alias="featureId")]
    """Feature identifier for the price override"""

    tiers: Iterable[PriceOverrideTier]
    """Pricing tiers configuration"""


class SubscriptionEntitlement(TypedDict, total=False):
    feature_id: Required[Annotated[str, PropertyInfo(alias="featureId")]]
    """Feature ID"""

    usage_limit: Required[Annotated[float, PropertyInfo(alias="usageLimit")]]

    is_granted: Annotated[bool, PropertyInfo(alias="isGranted")]


class TrialOverrideConfiguration(TypedDict, total=False):
    """Trial period override settings"""

    is_trial: Required[Annotated[bool, PropertyInfo(alias="isTrial")]]
    """Whether the subscription should start with a trial period"""

    trial_end_behavior: Annotated[
        Literal["CONVERT_TO_PAID", "CANCEL_SUBSCRIPTION"], PropertyInfo(alias="trialEndBehavior")
    ]
    """Behavior when trial ends: CONVERT_TO_PAID or CANCEL_SUBSCRIPTION"""

    trial_end_date: Annotated[Union[str, datetime], PropertyInfo(alias="trialEndDate", format="iso8601")]
    """Custom trial end date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import coupon_list_params, coupon_create_params, coupon_update_coupon_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.v1.coupon import Coupon
from ...types.v1.coupon_list_response import CouponListResponse

__all__ = ["CouponsResource", "AsyncCouponsResource"]


class CouponsResource(SyncAPIResource):
    """Operations related to coupons"""

    @cached_property
    def with_raw_response(self) -> CouponsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return CouponsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CouponsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return CouponsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        id: str,
        amounts_off: Optional[Iterable[coupon_create_params.AmountsOff]],
        description: Optional[str],
        duration_in_months: Optional[int],
        metadata: Optional[Dict[str, str]],
        name: str,
        percent_off: Optional[float],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Creates a new discount coupon with percentage or fixed amount off, applicable to
        customer subscriptions.

        Args:
          id: The unique identifier for the entity

          amounts_off: Fixed amount discounts in different currencies

          description: Description of the coupon

          duration_in_months: Duration of the coupon validity in months

          metadata: Metadata associated with the entity

          name: Name of the coupon

          percent_off: Percentage discount off the original price

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/coupons",
            body=maybe_transform(
                {
                    "id": id,
                    "amounts_off": amounts_off,
                    "description": description,
                    "duration_in_months": duration_in_months,
                    "metadata": metadata,
                    "name": name,
                    "percent_off": percent_off,
                },
                coupon_create_params.CouponCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Retrieves a coupon by its unique identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/v1/coupons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )

    def list(
        self,
        *,
        id: str | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: coupon_list_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        type: Literal["FIXED", "PERCENTAGE"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[CouponListResponse]:
        """
        Retrieves a paginated list of coupons in the environment.

        Args:
          id: Filter by entity ID

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          status: Filter by coupon status. Supports comma-separated values for multiple statuses

          type: Filter by coupon type (FIXED or PERCENTAGE)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/coupons",
            page=SyncMyCursorIDPage[CouponListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "status": status,
                        "type": type,
                    },
                    coupon_list_params.CouponListParams,
                ),
            ),
            model=CouponListResponse,
        )

    def archive_coupon(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Archives a coupon, preventing it from being applied to new subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/coupons/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )

    def update_coupon(
        self,
        id: str,
        *,
        description: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Updates an existing coupon's properties such as name, description, and metadata.

        Args:
          description: Description of the coupon

          metadata: Metadata associated with the entity

          name: Name of the coupon

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/coupons/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                    "name": name,
                },
                coupon_update_coupon_params.CouponUpdateCouponParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )


class AsyncCouponsResource(AsyncAPIResource):
    """Operations related to coupons"""

    @cached_property
    def with_raw_response(self) -> AsyncCouponsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCouponsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCouponsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncCouponsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        id: str,
        amounts_off: Optional[Iterable[coupon_create_params.AmountsOff]],
        description: Optional[str],
        duration_in_months: Optional[int],
        metadata: Optional[Dict[str, str]],
        name: str,
        percent_off: Optional[float],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Creates a new discount coupon with percentage or fixed amount off, applicable to
        customer subscriptions.

        Args:
          id: The unique identifier for the entity

          amounts_off: Fixed amount discounts in different currencies

          description: Description of the coupon

          duration_in_months: Duration of the coupon validity in months

          metadata: Metadata associated with the entity

          name: Name of the coupon

          percent_off: Percentage discount off the original price

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/coupons",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "amounts_off": amounts_off,
                    "description": description,
                    "duration_in_months": duration_in_months,
                    "metadata": metadata,
                    "name": name,
                    "percent_off": percent_off,
                },
                coupon_create_params.CouponCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Retrieves a coupon by its unique identifier.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/v1/coupons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )

    def list(
        self,
        *,
        id: str | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: coupon_list_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        status: str | Omit = omit,
        type: Literal["FIXED", "PERCENTAGE"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CouponListResponse, AsyncMyCursorIDPage[CouponListResponse]]:
        """
        Retrieves a paginated list of coupons in the environment.

        Args:
          id: Filter by entity ID

          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          status: Filter by coupon status. Supports comma-separated values for multiple statuses

          type: Filter by coupon type (FIXED or PERCENTAGE)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/coupons",
            page=AsyncMyCursorIDPage[CouponListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "status": status,
                        "type": type,
                    },
                    coupon_list_params.CouponListParams,
                ),
            ),
            model=CouponListResponse,
        )

    async def archive_coupon(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Archives a coupon, preventing it from being applied to new subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/coupons/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )

    async def update_coupon(
        self,
        id: str,
        *,
        description: Optional[str] | Omit = omit,
        metadata: Optional[Dict[str, str]] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Coupon:
        """
        Updates an existing coupon's properties such as name, description, and metadata.

        Args:
          description: Description of the coupon

          metadata: Metadata associated with the entity

          name: Name of the coupon

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/coupons/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "metadata": metadata,
                    "name": name,
                },
                coupon_update_coupon_params.CouponUpdateCouponParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Coupon,
        )


class CouponsResourceWithRawResponse:
    def __init__(self, coupons: CouponsResource) -> None:
        self._coupons = coupons

        self.create = to_raw_response_wrapper(
            coupons.create,
        )
        self.retrieve = to_raw_response_wrapper(
            coupons.retrieve,
        )
        self.list = to_raw_response_wrapper(
            coupons.list,
        )
        self.archive_coupon = to_raw_response_wrapper(
            coupons.archive_coupon,
        )
        self.update_coupon = to_raw_response_wrapper(
            coupons.update_coupon,
        )


class AsyncCouponsResourceWithRawResponse:
    def __init__(self, coupons: AsyncCouponsResource) -> None:
        self._coupons = coupons

        self.create = async_to_raw_response_wrapper(
            coupons.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            coupons.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            coupons.list,
        )
        self.archive_coupon = async_to_raw_response_wrapper(
            coupons.archive_coupon,
        )
        self.update_coupon = async_to_raw_response_wrapper(
            coupons.update_coupon,
        )


class CouponsResourceWithStreamingResponse:
    def __init__(self, coupons: CouponsResource) -> None:
        self._coupons = coupons

        self.create = to_streamed_response_wrapper(
            coupons.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            coupons.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            coupons.list,
        )
        self.archive_coupon = to_streamed_response_wrapper(
            coupons.archive_coupon,
        )
        self.update_coupon = to_streamed_response_wrapper(
            coupons.update_coupon,
        )


class AsyncCouponsResourceWithStreamingResponse:
    def __init__(self, coupons: AsyncCouponsResource) -> None:
        self._coupons = coupons

        self.create = async_to_streamed_response_wrapper(
            coupons.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            coupons.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            coupons.list,
        )
        self.archive_coupon = async_to_streamed_response_wrapper(
            coupons.archive_coupon,
        )
        self.update_coupon = async_to_streamed_response_wrapper(
            coupons.update_coupon,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from stigg import Stigg, AsyncStigg
from tests.utils import assert_matches_type
from stigg._utils import parse_datetime
from stigg.types.v1 import (
    Addon,
    AddonListResponse,
    AddonPublishResponse,
    AddonRemoveDraftResponse,
    SetPackagePricingResponse,
)
from stigg.pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAddons:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Stigg) -> None:
        addon = client.v1.addons.create(
            id="id",
            display_name="displayName",
            product_id="productId",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Stigg) -> None:
        addon = client.v1.addons.create(
            id="id",
            display_name="displayName",
            product_id="productId",
            billing_id="billingId",
            description="description",
            max_quantity=0,
            metadata={"foo": "string"},
            pricing_type="FREE",
            status="DRAFT",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.create(
            id="id",
            display_name="displayName",
            product_id="productId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.create(
            id="id",
            display_name="displayName",
            product_id="productId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Stigg) -> None:
        addon = client.v1.addons.retrieve(
            "x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Stigg) -> None:
        addon = client.v1.addons.update(
            id="x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Stigg) -> None:
        addon = client.v1.addons.update(
            id="x",
            billing_id="billingId",
            dependencies=["string"],
            description="description",
            display_name="displayName",
            max_quantity=0,
            metadata={"foo": "string"},
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.update(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.update(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Stigg) -> None:
        addon = client.v1.addons.list()
        assert_matches_type(SyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Stigg) -> None:
        addon = client.v1.addons.list(
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            limit=1,
            product_id="productId",
            status="status",
        )
        assert_matches_type(SyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(SyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(SyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_archive(self, client: Stigg) -> None:
        addon = client.v1.addons.archive(
            "x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_archive(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.archive(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_archive(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.archive(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_archive(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_draft(self, client: Stigg) -> None:
        addon = client.v1.addons.create_draft(
            "x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_draft(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.create_draft(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_draft(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.create_draft(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create_draft(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.create_draft(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_publish(self, client: Stigg) -> None:
        addon = client.v1.addons.publish(
            id="x",
            migration_type="NEW_CUSTOMERS",
        )
        assert_matches_type(AddonPublishResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_publish(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.publish(
            id="x",
            migration_type="NEW_CUSTOMERS",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(AddonPublishResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_publish(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.publish(
            id="x",
            migration_type="NEW_CUSTOMERS",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(AddonPublishResponse, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_publish(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.publish(
                id="",
                migration_type="NEW_CUSTOMERS",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove_draft(self, client: Stigg) -> None:
        addon = client.v1.addons.remove_draft(
            "x",
        )
        assert_matches_type(AddonRemoveDraftResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_remove_draft(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.remove_draft(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(AddonRemoveDraftResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_remove_draft(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.remove_draft(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(AddonRemoveDraftResponse, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_remove_draft(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.remove_draft(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_set_pricing(self, client: Stigg) -> None:
        addon = client.v1.addons.set_pricing(
            id="x",
            pricing_type="FREE",
        )
        assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_set_pricing_with_all_params(self, client: Stigg) -> None:
        addon = client.v1.addons.set_pricing(
            id="x",
            pricing_type="FREE",
            billing_id="billingId",
            minimum_spend=[
                {
                    "billing_period": "MONTHLY",
                    "minimum": {
                        "amount": 0,
                        "currency": "usd",
                    },
                }
            ],
            overage_billing_period="ON_SUBSCRIPTION_RENEWAL",
            overage_pricing_models=[
                {
                    "billing_model": "FLAT_FEE",
                    "price_periods": [
                        {
                            "billing_period": "MONTHLY",
                            "billing_country_code": "billingCountryCode",
                            "block_size": 0,
                            "credit_grant_cadence": "BEGINNING_OF_BILLING_PERIOD",
                            "credit_rate": {
                                "amount": 1,
                                "currency_id": "currencyId",
                                "cost_formula": "costFormula",
                            },
                            "price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "tiers": [
                                {
                                    "flat_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "unit_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "up_to": 0,
                                }
                            ],
                        }
                    ],
                    "billing_cadence": "RECURRING",
                    "entitlement": {
                        "feature_id": "featureId",
                        "has_soft_limit": True,
                        "has_unlimited_usage": True,
                        "monthly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                        "reset_period": "YEAR",
                        "usage_limit": 0,
                        "weekly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                        "yearly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    },
                    "feature_id": "featureId",
                    "top_up_custom_currency_id": "topUpCustomCurrencyId",
                }
            ],
            pricing_models=[
                {
                    "billing_model": "FLAT_FEE",
                    "price_periods": [
                        {
                            "billing_period": "MONTHLY",
                            "billing_country_code": "billingCountryCode",
                            "block_size": 0,
                            "credit_grant_cadence": "BEGINNING_OF_BILLING_PERIOD",
                            "credit_rate": {
                                "amount": 1,
                                "currency_id": "currencyId",
                                "cost_formula": "costFormula",
                            },
                            "price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "tiers": [
                                {
                                    "flat_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "unit_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "up_to": 0,
                                }
                            ],
                        }
                    ],
                    "billing_cadence": "RECURRING",
                    "feature_id": "featureId",
                    "max_unit_quantity": 1,
                    "min_unit_quantity": 1,
                    "monthly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "reset_period": "YEAR",
                    "tiers_mode": "VOLUME",
                    "top_up_custom_currency_id": "topUpCustomCurrencyId",
                    "weekly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "yearly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                }
            ],
        )
        assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_set_pricing(self, client: Stigg) -> None:
        response = client.v1.addons.with_raw_response.set_pricing(
            id="x",
            pricing_type="FREE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = response.parse()
        assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_set_pricing(self, client: Stigg) -> None:
        with client.v1.addons.with_streaming_response.set_pricing(
            id="x",
            pricing_type="FREE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = response.parse()
            assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_set_pricing(self, client: Stigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.v1.addons.with_raw_response.set_pricing(
                id="",
                pricing_type="FREE",
            )


class TestAsyncAddons:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.create(
            id="id",
            display_name="displayName",
            product_id="productId",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.create(
            id="id",
            display_name="displayName",
            product_id="productId",
            billing_id="billingId",
            description="description",
            max_quantity=0,
            metadata={"foo": "string"},
            pricing_type="FREE",
            status="DRAFT",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.create(
            id="id",
            display_name="displayName",
            product_id="productId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.create(
            id="id",
            display_name="displayName",
            product_id="productId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.retrieve(
            "x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.update(
            id="x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.update(
            id="x",
            billing_id="billingId",
            dependencies=["string"],
            description="description",
            display_name="displayName",
            max_quantity=0,
            metadata={"foo": "string"},
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.update(
            id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.update(
            id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.list()
        assert_matches_type(AsyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.list(
            after="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            before="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            created_at={
                "gt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "gte": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lt": parse_datetime("2019-12-27T18:11:19.117Z"),
                "lte": parse_datetime("2019-12-27T18:11:19.117Z"),
            },
            limit=1,
            product_id="productId",
            status="status",
        )
        assert_matches_type(AsyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(AsyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(AsyncMyCursorIDPage[AddonListResponse], addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_archive(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.archive(
            "x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.archive(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.archive(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_archive(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_draft(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.create_draft(
            "x",
        )
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_draft(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.create_draft(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(Addon, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_draft(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.create_draft(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(Addon, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create_draft(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.create_draft(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_publish(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.publish(
            id="x",
            migration_type="NEW_CUSTOMERS",
        )
        assert_matches_type(AddonPublishResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_publish(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.publish(
            id="x",
            migration_type="NEW_CUSTOMERS",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(AddonPublishResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_publish(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.publish(
            id="x",
            migration_type="NEW_CUSTOMERS",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(AddonPublishResponse, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_publish(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.publish(
                id="",
                migration_type="NEW_CUSTOMERS",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove_draft(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.remove_draft(
            "x",
        )
        assert_matches_type(AddonRemoveDraftResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_remove_draft(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.remove_draft(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(AddonRemoveDraftResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_remove_draft(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.remove_draft(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(AddonRemoveDraftResponse, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_remove_draft(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.remove_draft(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_set_pricing(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.set_pricing(
            id="x",
            pricing_type="FREE",
        )
        assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_set_pricing_with_all_params(self, async_client: AsyncStigg) -> None:
        addon = await async_client.v1.addons.set_pricing(
            id="x",
            pricing_type="FREE",
            billing_id="billingId",
            minimum_spend=[
                {
                    "billing_period": "MONTHLY",
                    "minimum": {
                        "amount": 0,
                        "currency": "usd",
                    },
                }
            ],
            overage_billing_period="ON_SUBSCRIPTION_RENEWAL",
            overage_pricing_models=[
                {
                    "billing_model": "FLAT_FEE",
                    "price_periods": [
                        {
                            "billing_period": "MONTHLY",
                            "billing_country_code": "billingCountryCode",
                            "block_size": 0,
                            "credit_grant_cadence": "BEGINNING_OF_BILLING_PERIOD",
                            "credit_rate": {
                                "amount": 1,
                                "currency_id": "currencyId",
                                "cost_formula": "costFormula",
                            },
                            "price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "tiers": [
                                {
                                    "flat_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "unit_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "up_to": 0,
                                }
                            ],
                        }
                    ],
                    "billing_cadence": "RECURRING",
                    "entitlement": {
                        "feature_id": "featureId",
                        "has_soft_limit": True,
                        "has_unlimited_usage": True,
                        "monthly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                        "reset_period": "YEAR",
                        "usage_limit": 0,
                        "weekly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                        "yearly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    },
                    "feature_id": "featureId",
                    "top_up_custom_currency_id": "topUpCustomCurrencyId",
                }
            ],
            pricing_models=[
                {
                    "billing_model": "FLAT_FEE",
                    "price_periods": [
                        {
                            "billing_period": "MONTHLY",
                            "billing_country_code": "billingCountryCode",
                            "block_size": 0,
                            "credit_grant_cadence": "BEGINNING_OF_BILLING_PERIOD",
                            "credit_rate": {
                                "amount": 1,
                                "currency_id": "currencyId",
                                "cost_formula": "costFormula",
                            },
                            "price": {
                                "amount": 0,
                                "currency": "usd",
                            },
                            "tiers": [
                                {
                                    "flat_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "unit_price": {
                                        "amount": 0,
                                        "currency": "usd",
                                    },
                                    "up_to": 0,
                                }
                            ],
                        }
                    ],
                    "billing_cadence": "RECURRING",
                    "feature_id": "featureId",
                    "max_unit_quantity": 1,
                    "min_unit_quantity": 1,
                    "monthly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "reset_period": "YEAR",
                    "tiers_mode": "VOLUME",
                    "top_up_custom_currency_id": "topUpCustomCurrencyId",
                    "weekly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                    "yearly_reset_period_configuration": {"according_to": "SubscriptionStart"},
                }
            ],
        )
        assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_set_pricing(self, async_client: AsyncStigg) -> None:
        response = await async_client.v1.addons.with_raw_response.set_pricing(
            id="x",
            pricing_type="FREE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        addon = await response.parse()
        assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_set_pricing(self, async_client: AsyncStigg) -> None:
        async with async_client.v1.addons.with_streaming_response.set_pricing(
            id="x",
            pricing_type="FREE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            addon = await response.parse()
            assert_matches_type(SetPackagePricingResponse, addon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_set_pricing(self, async_client: AsyncStigg) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.v1.addons.with_raw_response.set_pricing(
                id="",
                pricing_type="FREE",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AddonCreateParams"]


class AddonCreateParams(TypedDict, total=False):
    id: Required[str]
    """The unique identifier for the entity"""

    display_name: Required[Annotated[str, PropertyInfo(alias="displayName")]]
    """The display name of the package"""

    product_id: Required[Annotated[str, PropertyInfo(alias="productId")]]
    """The product id of the package"""

    billing_id: Annotated[Optional[str], PropertyInfo(alias="billingId")]
    """The unique identifier for the entity in the billing provider"""

    description: Optional[str]
    """The description of the package"""

    max_quantity: Annotated[Optional[int], PropertyInfo(alias="maxQuantity")]
    """The maximum quantity of this addon that can be added to a subscription"""

    metadata: Dict[str, str]
    """Metadata associated with the entity"""

    pricing_type: Annotated[Optional[Literal["FREE", "PAID", "CUSTOM"]], PropertyInfo(alias="pricingType")]
    """The pricing type of the package"""

    status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"]
    """The status of the package"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v1 import (
    V1Resource,
    AsyncV1Resource,
    V1ResourceWithRawResponse,
    AsyncV1ResourceWithRawResponse,
    V1ResourceWithStreamingResponse,
    AsyncV1ResourceWithStreamingResponse,
)
from .plans import (
    PlansResource,
    AsyncPlansResource,
    PlansResourceWithRawResponse,
    AsyncPlansResourceWithRawResponse,
    PlansResourceWithStreamingResponse,
    AsyncPlansResourceWithStreamingResponse,
)
from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .addons import (
    AddonsResource,
    AsyncAddonsResource,
    AddonsResourceWithRawResponse,
    AsyncAddonsResourceWithRawResponse,
    AddonsResourceWithStreamingResponse,
    AsyncAddonsResourceWithStreamingResponse,
)
from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .coupons import (
    CouponsResource,
    AsyncCouponsResource,
    CouponsResourceWithRawResponse,
    AsyncCouponsResourceWithRawResponse,
    CouponsResourceWithStreamingResponse,
    AsyncCouponsResourceWithStreamingResponse,
)
from .features import (
    FeaturesResource,
    AsyncFeaturesResource,
    FeaturesResourceWithRawResponse,
    AsyncFeaturesResourceWithRawResponse,
    FeaturesResourceWithStreamingResponse,
    AsyncFeaturesResourceWithStreamingResponse,
)
from .products import (
    ProductsResource,
    AsyncProductsResource,
    ProductsResourceWithRawResponse,
    AsyncProductsResourceWithRawResponse,
    ProductsResourceWithStreamingResponse,
    AsyncProductsResourceWithStreamingResponse,
)
from .customers import (
    CustomersResource,
    AsyncCustomersResource,
    CustomersResourceWithRawResponse,
    AsyncCustomersResourceWithRawResponse,
    CustomersResourceWithStreamingResponse,
    AsyncCustomersResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "CustomersResource",
    "AsyncCustomersResource",
    "CustomersResourceWithRawResponse",
    "AsyncCustomersResourceWithRawResponse",
    "CustomersResourceWithStreamingResponse",
    "AsyncCustomersResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
    "CouponsResource",
    "AsyncCouponsResource",
    "CouponsResourceWithRawResponse",
    "AsyncCouponsResourceWithRawResponse",
    "CouponsResourceWithStreamingResponse",
    "AsyncCouponsResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "FeaturesResource",
    "AsyncFeaturesResource",
    "FeaturesResourceWithRawResponse",
    "AsyncFeaturesResourceWithRawResponse",
    "FeaturesResourceWithStreamingResponse",
    "AsyncFeaturesResourceWithStreamingResponse",
    "AddonsResource",
    "AsyncAddonsResource",
    "AddonsResourceWithRawResponse",
    "AsyncAddonsResourceWithRawResponse",
    "AddonsResourceWithStreamingResponse",
    "AsyncAddonsResourceWithStreamingResponse",
    "PlansResource",
    "AsyncPlansResource",
    "PlansResourceWithRawResponse",
    "AsyncPlansResourceWithRawResponse",
    "PlansResourceWithStreamingResponse",
    "AsyncPlansResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
    "ProductsResource",
    "AsyncProductsResource",
    "ProductsResourceWithRawResponse",
    "AsyncProductsResourceWithRawResponse",
    "ProductsResourceWithStreamingResponse",
    "AsyncProductsResourceWithStreamingResponse",
    "V1Resource",
    "AsyncV1Resource",
    "V1ResourceWithRawResponse",
    "AsyncV1ResourceWithRawResponse",
    "V1ResourceWithStreamingResponse",
    "AsyncV1ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AddonPublishResponse", "Data"]


class Data(BaseModel):
    task_id: Optional[str] = FieldInfo(alias="taskId", default=None)
    """Task ID for tracking the async publish operation"""


class AddonPublishResponse(BaseModel):
    """Response containing task ID for publish operation"""

    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .payment_method_attach_params import PaymentMethodAttachParams as PaymentMethodAttachParams
from .promotional_entitlement_list_params import PromotionalEntitlementListParams as PromotionalEntitlementListParams
from .promotional_entitlement_create_params import (
    PromotionalEntitlementCreateParams as PromotionalEntitlementCreateParams,
)
from .promotional_entitlement_list_response import (
    PromotionalEntitlementListResponse as PromotionalEntitlementListResponse,
)
from .promotional_entitlement_create_response import (
    PromotionalEntitlementCreateResponse as PromotionalEntitlementCreateResponse,
)
from .promotional_entitlement_revoke_response import (
    PromotionalEntitlementRevokeResponse as PromotionalEntitlementRevokeResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CustomerListResourcesParams"]


class CustomerListResourcesParams(TypedDict, total=False):
    after: str
    """Return items that come after this cursor"""

    before: str
    """Return items that come before this cursor"""

    limit: int
    """Maximum number of items to return"""
